#!/usr/bin/python
# -*- encoding: utf-8 -*-
# cteni dat z magnetometru HMC5883 a korekce azimutu o naklon cidla

import smbus
import time
import math


bus = smbus.SMBus(1)  # novejsi varianta RasPi (512MB)
#bus = smbus.SMBus(0)  # starsi varianta RasPi (256MB)



#==============
# tento podprogram byl opsan z:
#       https://bitbucket.org/thinkbowl/i2clibraries/

#Returns heading in degrees and minutes
def getHeading(osa_1,osa_2):
		
  headingRad = math.atan2(osa_2, osa_1)
  headingRad += 3.5  * ( math.pi / 180)  # mistni posun magnetickeho polu vuci severnimu polu je 3,5 stupne

  # Correct for reversed heading
  if(headingRad < 0):
    headingRad += 2*math.pi
		
# Check for wrap and compensate
  if(headingRad > 2*math.pi):
    headingRad -= 2*math.pi
			
# Convert to degrees from radians
  headingDeg = headingRad * 180/math.pi
  degrees = int(math.floor(headingDeg))
  minutes = int(round(((headingDeg - degrees) * 60)))
  return (degrees, minutes, headingDeg)
#==============



# zakladni nastaveni magnetometru (I2C adresa obvodu je 0x1e)
bus.write_byte_data(0x1e,0x00,0x70) # prumerovani=8 vzorku ; rychlost vzorkovani = 15Hz
bus.write_byte_data(0x1e,0x01,0x00) # citlivost +/- 0.88 Ga (nejvyssi citlivost)
bus.write_byte_data(0x1e,0x02,0x00) # nepretrzite mereni

   
# zakladni nastaveni akcelerometru (naklonomeru)
bus.write_byte_data(0x68,0x6b,0x00) # reset a zacatek mereni (I2C adresa obvodu je 0x68)


while True:


  # cteni dat z magnetometru pomoci I2C sbernice
  data = bus.read_i2c_block_data(0x1e,0x03)
  mag_x = (data[0]*256) + data[1]             
  mag_z = (data[2]*256) + data[3]
  mag_y = (data[4]*256) + data[5]

  # prepocet pri zapornych hodnotach registru

  if(mag_x & 0xF000):        # pri zaporne hodnote jsou nejvyssi 4 bity ze 16 nastavene na "1"
    mag_x = mag_x & 0xFFF    # v tom pripade orezat nejvyssi 4 bity ...
    mag_x = -0x1000 + mag_x  # ... a spocitat dvojkovy doplnek

  if(mag_y & 0xF000):        # pri zaporne hodnote jsou nejvyssi 4 bity ze 16 nastavene na "1"
    mag_y = mag_y & 0xFFF    # v tom pripade orezat nejvyssi 4 bity ...
    mag_y = -0x1000 + mag_y  # ... a spocitat dvojkovy doplnek

  if(mag_z & 0xF000):        # pri zaporne hodnote jsou nejvyssi 4 bity ze 16 nastavene na "1"
    mag_z = mag_z & 0xFFF    # v tom pripade orezat nejvyssi 4 bity ...
    mag_z = -0x1000 + mag_z  # ... a spocitat dvojkovy doplnek

  # cteni dat z akcelerometru (naklonomeru) pomoci I2C sbernice

  # data ze vsech tri naklonomeru jsou v registrech 0x3b az 0x40
  data = bus.read_i2c_block_data(0x68,0x3b)
  nak_x = ((256 * data[0]) + data[1])
  nak_y = ((256 * data[2]) + data[3])
  nak_z = ((256 * data[4]) + data[5])

  # prepocet pri zapornych hodnotach registru
  if(nak_x & 0x8000):
    nak_x = -0x10000 + nak_x

  if(nak_y & 0x8000):
    nak_y = -0x10000 + nak_y

  if(nak_z & 0x8000):
    nak_z = -0x10000 + nak_z




# POZOR: obe cidla jsou pro nasledujici vypocty postavena "kolmo" k horizontalni rovine (viz fotka)

# otaceni naklonomeru v ose X v rozsahu -90 az +90 stupnu od vodorovne roviny
#    ma za nasledek zmenu hodnoty zrychleni v ose Z
#  (na modelu letadla je to "roll" - naklon leva/prava)

  nak_z_rad = math.atan2(nak_z , nak_y)
  roll = nak_z_rad 


# otaceni naklonomeru v ose Z v rozsahu -90 az +90 stupnu od vodorovne roviny
#    ma za nasledek zmenu hodnoty zrychleni v ose X 
#    (na modelu letadla je to "pitch" - naklon nahoru/dolu)

  nak_x_rad = math.atan2 (nak_y,nak_x) - math.pi / 2
  if (nak_x_rad < -(math.pi/2)) : nak_x_rad = nak_x_rad + math.pi
  pitch = nak_x_rad 



  Xh = ( mag_x * math.sin(pitch)) + (mag_y * math.cos(pitch))
  Yh = ( mag_x * math.cos(roll) * math.cos(pitch)) + (mag_z * math.sin(roll)) - (mag_y * math.cos(roll) * math.sin(pitch))

  natoceni = getHeading(Xh , Yh)
  aktualne = natoceni[2]          # v promenne natoceni[2] je azimut ve stupnich (desetinne cislo)
  print round(roll,3), "\t" ,round(pitch,3) , "\t" , mag_x ,  "\t" , mag_y  , "\t" , mag_z ,"\t" , int(aktualne)


  time.sleep(0.5)
  
  